import re
import unicodedata

BLANK = ' '

RE_OPS = re.I | re.M | re.S
RE_USR = re.compile(r"""@\S+""", RE_OPS)
RE_TAG = re.compile(r"""#\S+""", RE_OPS)
RE_URL = re.compile(r"""(http|ftp|https)://\S+""", RE_OPS)
RE_NUM = re.compile(r"""[-+]?\d+\.?\d*""", RE_OPS)

SYMBOLS_ = "()[]¿?¡!{}~<>|"
SYMBOLS = set(";:,.@\\-\"/" + SYMBOLS_)

def norm_chars(text):
    L = []
  
    for u in unicodedata.normalize('NFD', text):
        o = ord(u)
        if 0x300 <= o and o <= 0x036F:
            continue
           
        if u in ('\n', '\r', BLANK, '\t'):
            if len(L) == 0:
                continue

            u = BLANK
        
        if u in SYMBOLS:
            if len(L) > 0 and L[-1] != BLANK:
                L.append(BLANK)
            
            L.append(u)
            L.append(BLANK)
            continue
        
        L.append(u)

    return "".join(L)


def preprocess(text):
    text = RE_URL.sub("_url ", text)
    text = RE_USR.sub("_usr ", text)
    text = RE_TAG.sub("_htag ", text)
    text = RE_NUM.sub("0 ", text)
    text = re.sub(r"&amp;", "&", text)
    text = re.sub(r"&gt;", ">", text)
    text = re.sub(r"&lt;", "<", text)
    text = norm_chars(text.lower())
    text = re.sub(r"j(a|e|i)[jaei]+", r"j\1j\1", text)
    text = re.sub(r"h(a|e|i)[haei]+", r"j\1j\1", text)
    return re.sub(r"\s+", BLANK, text)


from bilma import bilma_model
import importlib_resources

class bilma_demo():
    def __init__(self, model_file):
        self.model, self.tokenizer = self.load_model(model_file)
    
    def pred(self, text, bs=10):
        n = len(text)
        emo = []
        st = 0
        for st in range(0, n, bs):
            toks = self.tokenizer.tokenize(text[st: st + bs])
            p = self.model.predict(toks)
            emo += self.tokenizer.decode_emo(p[1])
        
        return emo
    

    def load_model(self, model_file):
        
        model = bilma_model.load(model_file)
        my_resources = importlib_resources.files("bilma") / "resources"
        vocab_file = str((my_resources / "vocab_file_ALL.txt").joinpath())
        tokenizer = bilma_model.tokenizer(vocab_file="resources/vocab_file_ALL.txt", max_length=280)
        return model, tokenizer
    
    def emoticons(self):
        return self.tokenizer.emo_labels